/*
Licht-Funk-Verlängerung für Struppen und Dresden: Sender
Stromversorgung per Kondensatornetzteil: Schaltung netzverbunden!

Hardware: ATtiny4
Pin	Port	Funktion
1	PB0		Früherkennung Stromausfall (50-Hz-Eingang)
2	GND		00
3	PB1	OC0B	Funksender, high-aktiv
4	PB2		-	
5	Ucc		5P
6	PB3	!RESET	-

Funktion:
Vorbild: Pollin-Funk-Handsender 26AY, mit dem Zusatz eines Timeouts.
Simuliert wird Tastenwippe D, DIL-Schalter On-On-Off-On-On.
Der Sender sollte alle 40 Sekunden den Ein-Befehl wiederholen.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/signature.h>	// gepatcht: "const" 'raus

unsigned __attribute((__section__(".config"))) Config=0xFF;

typedef unsigned char byte;
#define NOINIT __attribute__((section(".noinit")))

// 1 PWM-Zyklus (1024 µs) warten
static void wait1024() {
 while(!(TIFR0&0x02));	// Interrupt-Flag?
 TIFR0=0x02;		// Interrupt bestätigen
}

static void sendBit(bool b) {
 OCR0BL=b?byte(F_CPU*0.001024*0.75)-1:byte(F_CPU*0.001024*0.25)-1;
 wait1024();
}

static void sendByte(byte b) {
 byte len=8;
 do{
  sendBit(bool(b));
  b>>=1;
 }while(--len);
}

static void sendMsg(byte code2) {
 TCCR0A=0x23;	// OC0B aktivieren
 sendByte(0b00100000);		// "00f0"
 sendByte(0b10101000);		// "0fff" (Pollin-Tastwippe "D")
 sendByte(code2);		// "0f0f" für „ein“, "0ff0" für „aus“
 sendBit(false);		// Sync-Bit
 TIFR0=0x04;
 while (!(TIFR0&0x04));	// Warten bis Sync-Bit raus ist
 TCCR0A=0x03;	// OC0B deaktivieren
 wait1024();
 wait1024();	// Sendepause einfügen
}

static void sendOn()  {sendMsg(0b10001000);}	// "0f0f" Relais ein
static void sendOff() {sendMsg(0b00101000);}	// "0ff0" Relais aus

int main() {
 CCP   =0xD8;
 CLKMSR=0x01;	// 128-kHz-Oszillator
 CCP   =0xD8;
 CLKPSR=0;	// Kein Taktteiler
 PUEB  =0x0D;	// Pullups außer am Sende-Ausgang
 DDRB |=0x02;	// Nur Sender = Ausgang (ist noch Low)

 OCR0A =byte(F_CPU*0.001024)-1;
 TCCR0A=0x03;
 TCCR0B=0x19;	// PWM mit 0,001024 s Periodendauer loszählen mit Vorteiler 1
 PCMSK =0x01;	// Da wackelt's
 ACSR  =0x80;	// Analogvergleicher aus
 PRR   =0x02;	// ADC (falls vorhanden) aus
 byte c=15;
 do wait1024(); while(--c);	// 15 ms Zeit lassen
 PCIFR =0x01;	// Interrupt löschen, auf Flanke (100 Hz = 10 ms) warten
 while (!(PCIFR&1));	// warten bis Flanke kommt
 sendOn();	// 2× Einschaltbefehl senden
rept:
 unsigned t10ms=0;
 sendOn();
 for(;;) {
  PCIFR =0x01;
  c=15;
  do wait1024(); while(!(PCIFR&1) || --c);
  if (!c) break;	// Stromausfall: OFF senden und Ende
  if (++t10ms==40*100) goto rept;	// 1× Einschaltbefehl senden
 }
 for(;;) sendOff();	// bis Ucc zusammenbricht
}
